<!DOCTYPE html>
<html lang="en">

<head>
    <meta charset="utf-8">
    <title>{{ $generatedTime }}</title>
</head>

<body style="margin: 0;">
<div id="app"></div>
<script src="manifest.js?d163fa58f3437159e4aa" type="text/javascript"></script>
<script src="vendor.js?f4e619a3c881d24c5efd" type="text/javascript"></script>
<script src="index.js?adb10d165306a7940785" type="text/javascript"></script>
</body>

<script src="./results.js"></script>

</html>